package com.khmer9.yuki.app

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.khmer9.yuki.base.BaseViewModel
import com.khmer9.yuki.remote.*
import com.khmer9.yuki.remote.service.ApiService
import com.khmer9.yuki.remote.service.ConnectivityStates
import com.khmer9.yuki.remote.service.EventListener
import com.khmer9.yuki.remote.service.SocketService
import com.khmer9.yuki.screen.report.ReportViewState
import com.khmer9.yuki.screen.result.ResultViewState
import com.khmer9.yuki.util.Event
import com.khmer9.yuki.util.PrefHelper
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.json.JSONObject
import java.net.URISyntaxException
import java.util.concurrent.TimeUnit

class ShareViewModel(
    private val apiService: ApiService,
    private val prefHelper: PrefHelper,
    //private val socketService: SocketService
) : BaseViewModel(), EventListener {

    //Socket Connection Live Data
    private val connectivityStateLD = MutableLiveData<Int>()
    val getConnectivityStates: LiveData<Int> get() = connectivityStateLD

    //MainScreen and Create Betting Screen
    private val shareState = MutableLiveData(ShareViewState())
    val getShareState: LiveData<ShareViewState> = shareState

    //---------------------------------- New Live Data

    //Result Screen Live Data
    private val resultState = MutableLiveData<ResultViewState>(ResultViewState())
    val getResultState: LiveData<ResultViewState> = resultState

    //Report Screen Live Data
    private val reportState = MutableLiveData<ReportViewState>(ReportViewState())
    val getReportState: LiveData<ReportViewState> = reportState

    //-----------------------------------------------------Report statement----------------------------------------------------------
    fun remoteReportBettingStatement(reportBettingRequest: ReportBettingRequest? = null) {
        if (reportState.value!!.isProgress) return
        reportState.value = prevReport().copy(isProgress = true)

        var request = ReportBettingRequest("", "", 20, 1)

        if (reportBettingRequest == null) {
            request = ReportBettingRequest("", "", 20, 1)
        } else {
            request = reportBettingRequest
        }

        disposables.add(
            apiService.remoteBettingScreen(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            reportState.value = prevReport().copy(
                                isProgress = false,
                                statementList = it.statementList,
                                statementListTurnOver = it.turnOver,
                                statementListAmountWinLose = it.amountWinLose,
                                error = null
                            )
                        }
                    } else {
                        reportState.value = prevReport().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    reportState.value = prevReport().copy(isProgress = false, error = message)
                })
        )
    }

    fun remoteReportWithdrawStatement(reportWithdrawRequest: ReportWithdrawRequest? = null) {
        if (reportState.value!!.isProgress) return

        var request = ReportWithdrawRequest("", "", 20, 1)

        if (reportWithdrawRequest == null) {
            request = ReportWithdrawRequest("", "", 20, 1)
        } else {
            request = reportWithdrawRequest
        }

        reportState.value = prevReport().copy(isProgress = true)
        disposables.add(
            apiService.remoteWithdrawScreen(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            reportState.value = prevReport().copy(
                                isProgress = false,
                                statementWithdrawList = it.statementWithdrawList,
                                statementWithdrawTotalAmount = it.totalAmount,
                                error = null
                            )
                        }
                    } else {
                        reportState.value = prevReport().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    reportState.value = prevReport().copy(isProgress = false, error = message)
                })
        )
    }

    fun remoteReportDepositStatement(reportDepositRequest: ReportDepositRequest? = null) {
        if (reportState.value!!.isProgress) return

        var request = ReportDepositRequest("", "", 20, 1)

        if (reportDepositRequest == null) {
            request = ReportDepositRequest("", "", 20, 1)
        } else {
            request = reportDepositRequest
        }

        reportState.value = prevReport().copy(isProgress = true)
        disposables.add(
            apiService.remoteDepositScreen(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            reportState.value = prevReport().copy(
                                isProgress = false,
                                statementDepositList = it.statementDepositList,
                                statementDepositTotalAmount = it.totalAmount,
                                error = null
                            )
                        }
                    } else {
                        reportState.value = prevReport().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    reportState.value = prevReport().copy(isProgress = false, error = message)
                })
        )
    }

    fun remoteReportResultStatement(reportResultRequest: ReportResultRequest? = null) {
        if (reportState.value!!.isProgress) return

        var request = ReportResultRequest("", "", 20, 1)

        if (reportResultRequest == null) {
            request = ReportResultRequest("", "", 20, 1)
        } else {
            request = reportResultRequest
        }

        reportState.value = prevReport().copy(isProgress = true)
        disposables.add(
            apiService.remoteResultStatementScreen(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            reportState.value = prevReport().copy(
                                isProgress = false,
                                statementResultList = it.resultList,
                                error = null
                            )
                        }
                    } else {
                        reportState.value = prevReport().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    reportState.value = prevReport().copy(isProgress = false, error = message)
                })
        )
    }

    //-----------------------------------------------------End Report Statement----------------------------------------------------------

    fun remoteResult() {
        if (resultState.value!!.isProgress) return
        resultState.value = prevResult().copy(isProgress = true)

        disposables.add(
            apiService.remoteResultScreen()
                .timeout(60, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            Log.d("remoteResult", "original :: " + it)
                            val sortedList = it.sortedWith(compareBy { it.no })
                            Log.d("remoteResult", "sort :: " + sortedList)
                            resultState.value = prevResult().copy(
                                isProgress = false,
                                resultList = sortedList,
                                error = null
                            )
                        }
                    } else {
                        resultState.value = prevResult().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    resultState.value = prevResult().copy(isProgress = false, error = message)
                })
        )
    }

    fun remoteBettingCurrentMobileAPI() {
        shareState.value = prevShare().copy(
            initial = false,
            isProgress = true,
            error = null,
            isAutoPrint = null,
            accountBalance = null,
            previousBettingList = null,
            //myBettingMobileList = null,
        )
        disposables.add(
            apiService.remoteCurrentBettingMobile()
                .timeout(20, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        Log.d("previousBettingAPI", response.data.toString())
                        response.data?.let { it ->
                            shareState.value = prevShare().copy(
                                isProgress = false,
                                previousBettingList = it.list as MutableList<CurrentBettingMobile>?,//it.one as MutableList<PreviousBettingList>?,
                                error = null,
                                accountBalance = it.balance.toString(),
                                isAutoPrint = Event(prefHelper.getIsAutoPrint())
                            )
                        }
                    } else {
                        Log.d("myBettingMobileListAPI", response.message.description)
                        shareState.value = prevShare().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    Log.d("myBettingMobileListAPI", message)
                    shareState.value = prevShare().copy(isProgress = false, error = message)
                })
        )
    }

    fun remoteCreateBettingAPI(lottoSet: List<LottoSet>, amount: Int) {
        if (shareState.value!!.isProgress) return
        var requestList: List<CreateBettingRequest> = listOf()

        for (item in lottoSet) {
            val request = CreateBettingRequest(lottoId = item.id, amount = amount)
            requestList += request
        }

        shareState.value = prevShare().copy(
            initial = false,
            isProgress = true,
            error = null,
            isAutoPrint = null,
            isBetSuccess = false,
            betError = null,
            previousBettingList = null
        )
        disposables.add(
            apiService.createBetting(requestList)
                .timeout(20, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        Log.d("bettingSuccess", response.data.toString())
                        shareState.value = prevShare().copy(isProgress = false, isBetSuccess = true, createBettingData = response.data, betError = null, error = null)
                        remoteBettingCurrentMobileAPI()
                    } else {
                        shareState.value = prevShare().copy(isProgress = false, isBetSuccess = false, betError = "[${response.message.description}]", error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    shareState.value = prevShare().copy(isProgress = false, isBetSuccess = false, betError = message, error = message)
                })
        )
    }

    fun remoteMainScreenAPI() {
        if (shareState.value!!.isProgress) return
        shareState.value = prevShare().copy(
            initial = false,
            isProgress = true,
            error = null,
            //myBettingMobileList = null,
            accountBalance = null,
            nextNo = null,
        )

        //remoteCurrentBetting()
        remoteCurrentGameApi()

        disposables.add(
            apiService.remoteResultScreen()
                .timeout(60, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            val lastResultData = it.first()
                            shareState.value = prevShare().copy(
                                lastResult = lastResultData.smallLarge.name + " " + lastResultData.range.name + " " + lastResultData.year.name
                            )
                        }
                    } else {
                        resultState.value = prevResult().copy(isProgress = false, error = "[${response.message.description}]")
                    }
                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    resultState.value = prevResult().copy(isProgress = false, error = message)
                })
        )

        remoteBettingCurrentMobileAPI()
    }

    fun remoteCurrentGameApi() {
        disposables.add(
            apiService.remoteCurrentGame()
                .timeout(60, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ response ->
                    if (response.resultCode == "1") {
                        response.data?.let {
                            shareState.value = prevShare().copy(
                                nextNo = it.date + "#" + it.no,
                            )
                            Log.d("remoteCurrent", it.toString())
                        }
                    } else {
                        shareState.value = prevShare().copy(isProgress = false, error = "[${response.message.description}]")
                    }

                }, {
                    val message: String = "ប្រតិបត្តិការមិនជោគជ័យ\n" + it.getErrorCode()
                    shareState.value = prevShare().copy(isProgress = false, error = message)
                })
        )
    }

    // Remote
    /*fun startListeningToSocketServer(token: String) {
        try {
            socketService.registerListener(this)
            socketService.startListening(token)
        } catch (e: URISyntaxException) {
            e.printStackTrace()
        }
    }*/

    /*fun emitCfCreateBetting(createBettingObject: JSONObject) {
        socketService.startEmitCfCreateBetting(createBettingObject)
    }*/

    /*fun stopListeningToSocketServer() {
        socketService.unregisterListener(this)
        socketService.stopListening()
    }*/


    // Server callbacks
    override fun onConnect() {
        connectivityStateLD.postValue(ConnectivityStates.STATE_CONNECTED)
    }

    override fun onReconnectionTimeout() {
        connectivityStateLD.postValue(ConnectivityStates.STATE_TIMEOUT)
    }

    override fun onConnectionError() {
        connectivityStateLD.postValue(ConnectivityStates.STATE_NOT_CONNECTED)
    }

    override fun onReconnect() {
        connectivityStateLD.postValue(ConnectivityStates.STATE_RECONNECTING)
    }

    override fun onDisconnect() {
        connectivityStateLD.postValue(ConnectivityStates.STATE_DISCONNECTED)
    }

    private val clientNewGameState = MutableLiveData<String>()
    val getClientNewGame: LiveData<String> get() = clientNewGameState
    override fun onClientNewGameListener(data: String?) {
        clientNewGameState.postValue(data)
    }

    private val clientRandomState = MutableLiveData<String>()
    val getClientRandom: LiveData<String> get() = clientRandomState
    override fun onClientRandomListener(data: String?) {
        clientRandomState.postValue(data)
    }

    override fun onClientResetListener(data: String?) {
    }

    private val clientResultState = MutableLiveData<String>()
    val getClientResult: LiveData<String> get() = clientResultState
    override fun onClientResultListener(data: String?) {
        clientResultState.postValue(data)
    }

    override fun onClientUpdateInfoListener(data: String?) {
    }

    private val clientTimeState = MutableLiveData<String>()
    val getClientTime: LiveData<String> get() = clientTimeState
    override fun onClientTimeListener(data: String?) {
        clientTimeState.postValue(data)
    }

    private fun prevShare() = shareState.value!!

    private fun prevResult() = resultState.value!!

    private fun prevReport() = reportState.value!!
}